#! /usr/bin/env python
"""Command line comleter helper for python.
"""

# Standard library imports.
from os import listdir
from pwd import getpwall
from os.path import split, expanduser
from rlcompleter import Completer

__date__ = "2024/01/07 19:58:42 hoel"
__author__ = "Sunjoong LEE <sunjoong@gmail.com>"
__copyright__ = """Copyright © 2006 by Sunjoong LEE
Copyright © 2020, 2022, 2024 by Berthold Höllmann"""
__credits__ = ["Sunjoong LEE", "Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


class IrlCompleter(Completer, object):
    def __init__(self, *args, **kw):
        super(IrlCompleter, self).__init__(*args, **kw)
        self.matches = ""

    def complete(self, text, state):
        if text == "":
            # you could replace '    ' to \t if you indent via tab
            return ["    ", None][state]
        if text.count("'") == 1:
            if not state:
                self.file_matches(text, "'")
            try:
                return self.matches[state]
            except IndexError:
                return None
        if text.count('"') == 1:
            if not state:
                self.file_matches(text, '"')
            try:
                return self.matches[state]
            except IndexError:
                return None
        else:
            return Completer.complete(self, text, state)

    def file_matches(self, text, mark):
        if "~" in text:
            if "/" in text:
                text = f'{mark}{expanduser(text[text.find("~") : text.find("/")])}{text[text.find("/") :]}'
            else:
                self.user_matches(text, mark)
                return

        text1 = text[1:]
        delim = "/"

        if not text1:
            directory = ""
        elif text1 == ".":
            directory = "."
        elif text1 == "..":
            directory = ".."
        elif text1 == "/":
            directory = "/"
            delim = ""
        elif text1.endswith("/"):
            directory = text1[:-1]
            delim = text1[len(directory) :]
        else:
            directory, partial = split(text1)
            delim = text1[len(directory) :][: -len(partial)]

        if directory:
            listing = [f"{mark}{directory}{delim}{x}" for x in listdir(directory)]
        else:
            listing = [f"{mark}{x}" for x in listdir(".")]

        n = len(text)
        self.matches = [x for x in listing if x[:n] == text]

    def user_matches(self, text, mark):
        self.matches = [f"{mark}~{x[0]}" for x in getpwall() if x.startswith(text)]



#! /usr/bin/env python
"""Handle command line history.
"""

# Standard library imports.
import sys
from pathlib import Path
from readline import (
    clear_history,
    set_completer,
    parse_and_bind,
    get_history_item,
    read_history_file,
    set_history_length,
    write_history_file,
    get_completer_delims,
    set_completer_delims,
    get_current_history_length,
)
from tempfile import mktemp
from itertools import count as icount

# Local library imports.
from .irl_completer import IrlCompleter

__date__ = "2024/01/07 20:00:15 hoel"
__author__ = "Sunjoong LEE <sunjoong@gmail.com>"
__copyright__ = """Copyright © 2006 by Sunjoong LEE
Copyright © 2020, 2022, 2024 Berthold Höllmann"""
__credits__ = ["Sunjoong LEE", "Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"

HISTORY_PATH = Path.home() / f".pyhistory{sys.version_info.major}"
HISTORY_LENGTH = 100


class History:
    def __init__(self):
        self.recall()
        set_history_length(HISTORY_LENGTH)

        parse_and_bind("tab: complete")
        delims = get_completer_delims()
        set_completer_delims(delims)
        set_completer(IrlCompleter().complete)

    def __repr__(self):
        """print out current history information"""
        length = get_current_history_length()
        if length > 1:
            try:
                return "\n".join(get_history_item(i) for i in range(1, length))
            except UnicodeDecodeError:
                return b"\n".join(get_history_item(i) for i in range(1, length))
        else:
            return ""

    def __call__(self):
        """print out current history information with line number"""
        length = get_current_history_length()
        if length > 1:
            kount = icount(1)
            for command in [get_history_item(i) for i in range(1, length)]:
                try:
                    print("{}\t{}".format(next(kount), command))
                except UnicodeDecodeError:
                    print(b"{}\t{}".format(next(kount), command))

    @staticmethod
    def save(filename, pos=None, end=None):
        """write history number from pos to end into filename file"""
        length = get_current_history_length()
        if length > 1:
            if not pos:
                pos = 1
            elif pos >= length - 1:
                pos = length - 1
            elif pos < 1:
                pos = length + pos - 1
            if not end:
                end = length
            elif end >= length:
                end = length
            if end < 0:
                end = length + end
            else:
                end = end + 1

            with open(filename, "w") as f_p:
                if pos < end:
                    for i in range(pos, end):
                        f_p.write("{}\n".format(get_history_item(i)))
                else:
                    f_p.write("{}\n".format(get_history_item(pos)))

    @staticmethod
    def clear():
        """save the current history and clear it"""
        write_history_file(HISTORY_PATH)
        clear_history()

    @staticmethod
    def recall(history_path=HISTORY_PATH):
        """clear the current history and recall it from saved"""
        clear_history()
        if history_path.exists():
            read_history_file(history_path)

    @staticmethod
    def execute(pos, end=None):
        """execute history number from pos to end"""
        length = get_current_history_length()
        if length > 1:
            if pos >= length - 1:
                pos = length - 1
            elif pos < 1:
                pos = length + pos - 1
            if not end:
                end = pos + 1
            elif end >= length:
                end = length
            if end < 0:
                end = length + end
            else:
                end = end + 1

            filename = Path(mktemp())
            with open(filename, "w") as f_p:
                for i in range(pos, end):
                    f_p.write(f"{get_history_item(i)}\n")
            try:
                with open(filename, "rb") as f_p:
                    exec(compile(f_p.read(), filename, "exec"), locals())
                read_history_file(filename)
            except Exception:
                pass
            filename.unlink()



#! /usr/bin/env python
"""Save command line history and provide a command line completer for python.
"""

# Standard library imports.
from atexit import register
from readline import set_pre_input_hook

# Local library imports.
from .history import History

__date__ = "2024/01/07 20:00:16 hoel"
__author__ = "Sunjoong LEE <sunjoong@gmail.com>"
__copyright__ = """Copyright © 2006 by Sunjoong LEE
Copyright © 2020, 2022, 2024 Berthold Höllmann"""
__credits__ = ["Sunjoong LEE", "Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


__version__ = __import__("importlib.metadata", fromlist=["version"]).version(
    "pyCmdLineHistory"
)


def save_history(history_path=None):
    # Standard library imports.
    from readline import write_history_file

    # Local library imports.
    from .history import HISTORY_PATH

    if history_path is None:
        history_path = HISTORY_PATH
    write_history_file(history_path)


register(save_history)


def hook():
    # Standard library imports.
    from readline import set_pre_input_hook

    set_pre_input_hook()
    del locals()["History"]
    del locals()["__file__"]


set_pre_input_hook(hook)

locals()["__builtins__"]["history"] = History()



#! /usr/bin/env python
"""Tests for `cmd_line_history`.
"""

# Standard library imports.
from pathlib import Path

# Third party library imports.
import toml
import pytest

# First party library imports.
from berhoel import cmd_line_history

__date__ = "2024/01/07 20:00:16 hoel"
__author__ = "Berthold Höllmann"
__copyright__ = "Copyright © 2020, 2024 by Berthold Höllmann"
__credits__ = ["Berthold Höllmann"]
__maintainer__ = "Berthold Höllmann"
__email__ = "berhoel@gmail.com"


@pytest.fixture
def base_path():
    result = Path(__file__).parent
    while not (result / "pyproject.toml").is_file():
        result = result.parent
    return result


@pytest.fixture
def config(base_path):
    return base_path / "pyproject.toml"


@pytest.fixture
def toml_data(config):
    return toml.load(config.open("r"))


def test_version(toml_data):
    """Testing for consistent version number."""
    assert cmd_line_history.__version__ == toml_data["tool"]["poetry"]["version"]
